@@ -4,10 +4,17 @@ import errno
 import fcntl
 import os
 import os.path
+import platform
 import select
 import time
 
 
+try:
+    KERNEL_VERSION = tuple([int(s) for s in platform.release().split(".")[:2]])
+except ValueError:
+    KERNEL_VERSION = (0, 0)
+
+
 class GPIOError(IOError):
     """Base class for GPIO errors."""
     pass
@@ -175,7 +182,7 @@ class GPIO(object):
         return results
 
     def close(self):
-        """Close the sysfs GPIO.
+        """Close the GPIO.
 
         Raises:
             GPIOError: if an I/O or OS error occurs.
@@ -439,6 +446,8 @@ class CdevGPIO(GPIO):
     _GPIOEVENT_EVENT_RISING_EDGE = 0x1
     _GPIOEVENT_EVENT_FALLING_EDGE = 0x2
 
+    _SUPPORTS_LINE_BIAS = KERNEL_VERSION >= (5, 5)
+
     def __init__(self, path, line, direction, edge="none", bias="default", drive="default", inverted=False, label=None):
         """**Character device GPIO**
 
@@ -498,22 +507,22 @@ class CdevGPIO(GPIO):
 
         if not isinstance(direction, str):
             raise TypeError("Invalid direction type, should be string.")
-        elif direction.lower() not in ["in", "out", "high", "low"]:
+        elif direction not in ["in", "out", "high", "low"]:
             raise ValueError("Invalid direction, can be: \"in\", \"out\", \"high\", \"low\".")
 
         if not isinstance(edge, str):
             raise TypeError("Invalid edge type, should be string.")
-        elif edge.lower() not in ["none", "rising", "falling", "both"]:
+        elif edge not in ["none", "rising", "falling", "both"]:
             raise ValueError("Invalid edge, can be: \"none\", \"rising\", \"falling\", \"both\".")
 
         if not isinstance(bias, str):
             raise TypeError("Invalid bias type, should be string.")
-        elif bias.lower() not in ["default", "pull_up", "pull_down", "disable"]:
+        elif bias not in ["default", "pull_up", "pull_down", "disable"]:
             raise ValueError("Invalid bias, can be: \"default\", \"pull_up\", \"pull_down\", \"disable\".")
 
         if not isinstance(drive, str):
             raise TypeError("Invalid drive type, should be string.")
-        elif drive.lower() not in ["default", "open_drain", "open_source"]:
+        elif drive not in ["default", "open_drain", "open_source"]:
             raise ValueError("Invalid drive, can be: \"default\", \"open_drain\", \"open_source\".")
 
         if not isinstance(inverted, bool):
@@ -540,7 +549,9 @@ class CdevGPIO(GPIO):
     def _reopen(self, direction, edge, bias, drive, inverted):
         flags = 0
 
-        if bias == "pull_up":
+        if bias != "default" and not CdevGPIO._SUPPORTS_LINE_BIAS:
+            raise GPIOError(None, "Line bias configuration not supported by kernel version {}.{}.".format(*KERNEL_VERSION))
+        elif bias == "pull_up":
             flags |= CdevGPIO._GPIOHANDLE_REQUEST_BIAS_PULL_UP
         elif bias == "pull_down":
             flags |= CdevGPIO._GPIOHANDLE_REQUEST_BIAS_PULL_DOWN
@@ -820,7 +831,7 @@ class CdevGPIO(GPIO):
     def _set_direction(self, direction):
         if not isinstance(direction, str):
             raise TypeError("Invalid direction type, should be string.")
-        if direction.lower() not in ["in", "out", "high", "low"]:
+        if direction not in ["in", "out", "high", "low"]:
             raise ValueError("Invalid direction, can be: \"in\", \"out\", \"high\", \"low\".")
 
         if self._direction == direction:
@@ -836,7 +847,7 @@ class CdevGPIO(GPIO):
     def _set_edge(self, edge):
         if not isinstance(edge, str):
             raise TypeError("Invalid edge type, should be string.")
-        if edge.lower() not in ["none", "rising", "falling", "both"]:
+        if edge not in ["none", "rising", "falling", "both"]:
             raise ValueError("Invalid edge, can be: \"none\", \"rising\", \"falling\", \"both\".")
 
         if self._direction != "in":
@@ -855,7 +866,7 @@ class CdevGPIO(GPIO):
     def _set_bias(self, bias):
         if not isinstance(bias, str):
             raise TypeError("Invalid bias type, should be string.")
-        if bias.lower() not in ["default", "pull_up", "pull_down", "disable"]:
+        if bias not in ["default", "pull_up", "pull_down", "disable"]:
             raise ValueError("Invalid bias, can be: \"default\", \"pull_up\", \"pull_down\", \"disable\".")
 
         if self._bias == bias:
@@ -871,7 +882,7 @@ class CdevGPIO(GPIO):
     def _set_drive(self, drive):
         if not isinstance(drive, str):
             raise TypeError("Invalid drive type, should be string.")
-        if drive.lower() not in ["default", "open_drain", "open_source"]:
+        if drive not in ["default", "open_drain", "open_source"]:
             raise ValueError("Invalid drive, can be: \"default\", \"open_drain\", \"open_source\".")
 
         if self._direction != "out" and drive != "default":
